@@ -12,27 +12,32 @@ world <- downloadCovid19("world")
 
 
 # adding the geometry/epidemiological rates to the data:
-#regions <- readRDS("regions.rds")
-regions_geo <- regions %>%
-  filter(date == max(date)) %>%
-  add_geo() %>%
-  add_epi_rates()
 
-#states <- readRDS("states.rds")
+if(nrow(regions)>0){
+  regions_geo <- regions %>%
+    filter(date == max(date)) %>%
+    add_geo() %>%
+    add_epi_rates()
+}
+
+if(nrow(states)>0){
 states_geo <- states %>%
   filter(date == max(date)) %>%
   add_geo() %>%
   add_epi_rates()
+}
 
-#cities <- readRDS("cities.rds")
+if(nrow(cities)>0){
 cities_geo <- cities %>%
   filter(date == max(date)) %>%
   add_geo() %>%
   add_epi_rates()
+}
 
+if(nrow(world)>0){
 world_geo <- world %>%
   filter(date == max(date)) %>%
   add_geo() %>%
   add_epi_rates()
-
+}
 
